import java.util.HashSet;
import java.util.Set;
/**
 * Object class. An instance represents a network.
 * @author Zhihao
 *
 */
class Network {
	
	Node[] nodes; 		//nodes in network
	Arc[] arcs;			//arcs in network
	int[] supplyNodes;	//list of supply nodes
	int[] transitNodes;	//list of transit nodes
	int[] demandNodes;	//list of demand nodes
	
	Network(String[] nodeNames,int[] supplyNodes_,int[] demandNodes_,double[] penalties,int[][] arcIndices,double[] capCosts,double[] flowCosts) {
		supplyNodes = supplyNodes_;
		demandNodes = demandNodes_;
		
		Set<Integer> supplySet = new HashSet<Integer>();
		for(int i = 0; i < supplyNodes.length; i++) {
			supplySet.add(supplyNodes[i]);
		}
		Set<Integer> demandSet = new HashSet<Integer>();
		for(int i = 0; i < demandNodes.length; i++) {
			demandSet.add(demandNodes[i]);
		}
		Set<Integer> transitSet = new HashSet<Integer>();
		for(int i = 0; i < nodeNames.length; i++) {
			transitSet.add(i);
		}
		transitSet.removeAll(supplySet);
		transitSet.removeAll(demandSet);
		Integer[] temp = transitSet.toArray(new Integer[transitSet.size()]);
		transitNodes = new int[temp.length];
		for(int i = 0; i < transitNodes.length; i++) {
			transitNodes[i] = temp[i];
		}
		
		nodes = new Node[nodeNames.length];
		for(int i = 0; i < supplyNodes.length; i++) {
			nodes[supplyNodes[i]] = new Node(nodeNames[supplyNodes[i]],Node.SUPPLY,0);
		}
		for(int i = 0; i < transitNodes.length; i++) {
			nodes[transitNodes[i]] = new Node(nodeNames[transitNodes[i]],Node.TRANSIT,0);
		}
		for(int i = 0; i < demandNodes.length; i++) {
			nodes[demandNodes[i]] = new Node(nodeNames[demandNodes[i]],Node.DEMAND,penalties[i]);
		}
		
		arcs = new Arc[arcIndices.length];
		for(int ij = 0; ij < arcs.length; ij++) {
			arcs[ij] = new Arc(arcIndices[ij][0],arcIndices[ij][1],capCosts[ij],flowCosts[ij]);
			nodes[arcIndices[ij][0]].arcsOut.add(ij);
			nodes[arcIndices[ij][1]].arcsIn.add(ij);
		}
	}
	/**
	 * Displays the network in text form. Includes (only) the number of nodes,
	 * the number of arcs, and the list of arcs.
	 */
	void displayTextNetwork() {
		System.out.println("NUMBER OF NODES: "+this.nodes.length);
		System.out.println("NUMBER OF ARCS: "+this.arcs.length);
		System.out.println("LIST OF SUPPLY NODES:");
		for(int i = 0; i < supplyNodes.length; i++) {
			System.out.println(nodes[supplyNodes[i]].name);
		}
		System.out.println("LIST OF DEMAND NODES:");
		for(int i = 0; i < demandNodes.length; i++) {
			System.out.println(nodes[demandNodes[i]].name);
		}
		System.out.println("LIST OF TRANSIT NODES:");
		for(int i = 0; i < transitNodes.length; i++) {
			System.out.println(nodes[transitNodes[i]].name);
		}
		System.out.println("LIST OF ARCS:");
		for(int ij = 0; ij < arcs.length; ij++) {
			System.out.println(arcs[ij].asString()
					+"\n  Capacity Cost: "+arcs[ij].capCost
					+"\n  Flow cost: "+arcs[ij].flowCost
					+"\n----------");
		}
	}
}
