import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
/**
 * Static helper class. Reads from and writes to files
 * @author Zhihao
 *
 */
class ReaderWriter {
	/**
	 * Constructs a network from a text file
	 * @param filename
	 * @return
	 */
	static Network file2network(String filename) {
		try {
			BufferedReader br = new BufferedReader(new FileReader(filename));
			//skip "Number of nodes, supply nodes, demand nodes, arcs:" line
			br.readLine();
			//read network sizes
			String[] splitln = br.readLine().split(",");
			int numNodes = Integer.parseInt(splitln[0]);
			int numSupplyNodes = Integer.parseInt(splitln[1]);
			int numDemandNodes = Integer.parseInt(splitln[2]);
			int numArcs = Integer.parseInt(splitln[3]);
			//initialize network parameters
			String[] nodeNames = new String[numNodes];
			int[] supplyNodes = new int[numSupplyNodes];
			int[] demandNodes = new int[numDemandNodes];
			double[] penalties = new double[numDemandNodes];
			int[][] arcIndices = new int[numArcs][2];
			double[] capCosts = new double[numArcs];
			double[] flowCosts = new double[numArcs];
			//skip blank line
			br.readLine();
			//skip "Nodes:" line
			br.readLine();
			//read node names
			for(int i = 0; i < numNodes; i++) {
				nodeNames[i] = br.readLine().split("-")[1];
			}
			//skip blank line
			br.readLine();
			//skip "Supply node indices:" line
			br.readLine();
			//read supply node indices
			for(int i = 0; i < numSupplyNodes; i++) {
				supplyNodes[i] = Integer.parseInt(br.readLine());
			}
			//skip blank line
			br.readLine();
			//skip "Demand node indices, penalties:" line
			br.readLine();
			//read demand node indices and their penalties
			for(int i = 0; i < numDemandNodes; i++) {
				splitln = br.readLine().split(",");
				demandNodes[i] = Integer.parseInt(splitln[0]);
				penalties[i] = Double.parseDouble(splitln[1]);
			}
			//skip blank line
			br.readLine();
			//skip "Arcs, capacity costs, flow costs:" line
			br.readLine();
			//read arcs: origin and destination indices, and capacity and flow costs
			for(int ij = 0; ij < numArcs; ij++) {
				splitln = br.readLine().split(",");
				arcIndices[ij][0] = Integer.parseInt(splitln[0]);
				arcIndices[ij][1] = Integer.parseInt(splitln[1]);
				capCosts[ij] = Double.parseDouble(splitln[2]);
				flowCosts[ij] = Double.parseDouble(splitln[3]);
			}
			br.close();
			return new Network(nodeNames,supplyNodes,demandNodes,penalties,arcIndices,capCosts,flowCosts);
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
			return null;
		}
	}
	static Instance file2instance(Network network,String filename) {
		try {
			BufferedReader br = new BufferedReader(new FileReader(filename));
			//skip "Supplies:" line
			br.readLine();
			//read supplies
			String[] splitln = br.readLine().split(",");
			double[] supplies = new double[network.supplyNodes.length];
			for(int i = 0; i < network.supplyNodes.length; i++) {
				supplies[i] = Double.parseDouble(splitln[i]);
			}
			//skip blank line
			br.readLine();
			//skip "Demand lower bounds:" line
			br.readLine();
			//read demand lower bounds
			splitln = br.readLine().split(",");
			double[] demandLBs = new double[network.demandNodes.length];
			for(int i = 0; i < network.demandNodes.length; i++) {
				demandLBs[i] = Double.parseDouble(splitln[i]);
			}
			//skip blank line
			br.readLine();
			//skip "Demand upper bounds:" line
			br.readLine();
			//read demand upper bounds
			splitln = br.readLine().split(",");
			double[] demandUBs = new double[network.demandNodes.length];
			for(int i = 0; i < network.demandNodes.length; i++) {
				demandUBs[i] = Double.parseDouble(splitln[i]);
			}
			//skip blank line
			br.readLine();
			//skip "Number of samples:" line
			br.readLine();
			//read number of samples
			int numSamples = Integer.parseInt(br.readLine());
			//skip blank line
			br.readLine();
			//skip "Demand samples:" line
			br.readLine();
			//read demand samples
			double[][] samples = new double[numSamples][network.demandNodes.length];
			for(int s = 0; s < numSamples; s++) {
				splitln = br.readLine().split(",");
				for(int i = 0; i < network.demandNodes.length; i++) {
					samples[s][i] = Double.parseDouble(splitln[i]);
				}
			}
			br.close();
			return new Instance(supplies,demandLBs,demandUBs,samples);
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
			return null;
		}
	}
	static void network2file(Network network,String filename) {
		try {
			PrintWriter writer = new PrintWriter(filename);
			writer.println("Number of nodes, supply nodes, demand nodes, arcs:");
			writer.printf("%d,%d,%d,%d\n",network.nodes.length,network.supplyNodes.length,network.demandNodes.length,network.arcs.length);
			writer.println();
			writer.println("Nodes:");
			for(int i = 0; i < network.nodes.length; i++) {
				writer.printf("%d-%s\n",i,network.nodes[i].name);
			}
			writer.println();
			writer.println("Supply node indices:");
			for(int i = 0; i < network.supplyNodes.length; i++) {
				writer.printf("%d\n",network.supplyNodes[i]);
			}
			writer.println();
			writer.println("Demand node indices, penalties:");
			for(int i = 0; i < network.demandNodes.length; i++) {
				writer.printf("%d,%.2f\n",network.demandNodes[i],network.nodes[network.demandNodes[i]].penalty);
			}
			writer.println();
			writer.println("Arcs, capacity costs, flow costs:");
			for(int ij = 0; ij < network.arcs.length; ij++) {
				writer.printf("%d,%d,%.2f,%.2f\n",network.arcs[ij].origin,network.arcs[ij].destination,network.arcs[ij].capCost,network.arcs[ij].flowCost);
			}
			writer.close();
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
	static void instance2file(Instance instance,String filename) {
		try {
			PrintWriter writer = new PrintWriter(filename);
			writer.println("Supplies:");
			for(int i = 0; i < instance.supplies.length; i++) {
				writer.printf("%.0f,",instance.supplies[i]);
			}
			writer.printf("\n");
			writer.println();
			writer.println("Demand lower bounds:");
			for(int i = 0; i < instance.demandLBs.length; i++) {
				writer.printf("%.0f,",instance.demandLBs[i]);
			}
			writer.printf("\n");
			writer.println();
			writer.println("Demand upper bounds:");
			for(int i = 0; i < instance.demandUBs.length; i++) {
				writer.printf("%.0f,",instance.demandUBs[i]);
			}
			writer.printf("\n");
			writer.println();
			writer.println("Number of samples:");
			writer.println(instance.scenarios.length);
			writer.println();
			writer.println("Scenarios:");
			for(int s = 0; s < instance.scenarios.length; s++) {
				for(int i = 0; i < instance.scenarios[s].length; i++) {
					writer.printf("%.0f,",instance.scenarios[s][i]);
				}
				writer.printf("\n");
			}
			writer.close();
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
}
