# DRNDP
Network design in scarce data environment using moment-based distributionally robust optimization

https://www.sciencedirect.com/science/article/pii/S0305054817301661

############################## COPYRIGHT ##############################

All rights reserved. The authorized use of this data/code is limited to informational and educational purposes only, and NOT for operational or commercial purposes. No part of this zip file including all the data generation files, example data output and code files may be reproduced, distributed, or transmitted in any form or by any means, without the prior written permission of Prof. Siqian Shen (siqian@umich.edu). For permission requests, write to Siqian Shen (siqian@umich.edu), with a subject “Attention: Permissions for Using DRNDP Data.”

########################################################################

To run it, there are several steps:
1) (Optional) Generate a network from graph files saved under "DRNDP/graphs" or grid network
2) Load a network from a text file with the ReaderWriter class.
3) Load an instance from a text file of data (reference set) with the ReaderWriter class.
4) Generate (several) samples randomly picked from the reference set using the SampleMaker class. Otherwise, load the samples from a text file with the ReaderWriter class.
5) Solve with one of the solver classes DRNDP, RNDP, SNDP to obtain an Output object.
6) Write the outputs from the Output object to a text file using the appropriate print function in the Output class.

======================== files =========================================

Data classes
- Network
	- Node.java
	- Arc.java
	- Network.java
- Data/output
	- Instance.java: Contains a set of reference set data
	- SampleMaker.java: Generates random samples from reference set data (instance)
	- Output.java: Wrapper class for ALL multi-output function (in particular the solver functions)
- Deprecated
	- NetworkSizeReducer.java: Randomly selects some supply and demand nodes and creates the sub-network and sub-instance associated with the subset of nodes
	
Functional classes
- Solvers (Each object contains the inputs required to solve the associated problem, and contains the solve function)
	- NDP
	- DRNDP
	- SNDP
	- RNDP
- Helpers
	- ReaderWriter: Contains read and write functions for networks and instances
	- NewFolder: Creates new directories
	- Sort: Sorts arrays
- Miscellaneous
	- Main: This is where you run the methods you want to run
		- As an example, we have made samples which 
			- 1. call network files and compute on a default parameter
			- 2. refer to the above but change the supply/demand location
			- 3. vary penalty factor
			- 4. vary number of intervals
			- 5. vary number of demand locations
		- The parameters are as in Section 5.

======================== input/output files =========================================

datafiles_and_results/Example1/Instance0/

	├── nobel-us				//name of graph
	│   ├── input\ files				
	│   │   ├── network.csv			//graph information
	│   │   ├── reference\ set.csv		//reference data
	│   │   ├── sample\ 1			//normal demand
	│   │   │   ├── drndp.csv
	│   │   │   ├── gamma50.csv		//gamma: distribution(words), 50: #demand samples
	│   │   │   ├── ...
	│   │   │   └── meanVar.csv		//mean and variance assumed in "sample folder"
	│   │   ├── sample\ 2			//low demand
	│   │   │   └── ...
	│   │   └── sample\ 3			//very low demand
	│   │       └── ...
	│   └── output\ files
	│       ├── sample\ 1			//normal demand
	│       │   ├── drndp
	│       │   │   ├── currentX.csv		//X
	│       │   │   ├── ...
	│       │   │   ├── summary.csv			//X,solveTime,optVal,capCost,secondStageSol,numCuts,htime
	│       │   │   └── performance\ summary.csv	//X,capCost+meanFlowCost+meanPenaltyCost,capCost,meanFlowCost,meanPenaltyCost
	│       │   ├── gamma1000
	│       │   │   └── ..
	│       │   ├── ..
	│       │   └── rndp_0.4			//robust NDP with cardinality 0.4
	│       │       └── ...
	│       ├── sample\ 2
	│       │   └── ...
	│       └── sample\ 3
	│           └── ...
	└── summarized\ means.csv		//Summary of summaries in output files
